@@ -106,18 +106,9 @@ function(x, y)
         Vectorize(.help)(x, y)
 }
 
-"%e%" <-
-function(e, x) {
-    if (is.set(x))
-        set_contains_element(x, e)
-    else
-        gset_contains_element(x, e)
-}
-
-
 gset_contains_element <-
 function(x, e)
-    set_contains_element(.make_list_of_elements_from_gset(x), e(e))
+    set_contains_element(as.set(.make_list_of_elements_from_cset(x)), e(e))
 
 gset_is_multiset <-
 function(x)
@@ -159,3 +150,103 @@ function(x)
     !is.list(m) && all(m == 1)
 }
 
+## cset-predicates
+
+is.cset <-
+function(x)
+    inherits(x, c("cset", "gset", "set"))
+
+cset_is_empty <-
+function(x)
+{
+    if(is.cset(x))
+        length(x) == 0
+    else
+        sapply(x, length) == 0
+}
+
+cset_is_subset <-
+function(x, y)
+{
+    .help <- function(a, b) cset_is_empty(cset_complement(b, a))
+    if(is.cset(x))
+        .help(x, y)
+    else
+        Vectorize(.help)(x, y)
+}
+
+cset_is_proper_subset <-
+function(x, y)
+{
+    cset_is_subset(x, y) &
+    if(is.cset(x))
+        length(x) != length(y)
+    else
+        sapply(x, length) != sapply(y, length)
+}
+
+cset_is_equal <-
+function(x, y)
+{
+    .help <- function(a, b)
+        ((length(a) == length(b))
+         && (length(cset_intersection(a,b)) == length(a)))
+    if(is.cset(x))
+        .help(x, y)
+    else
+        Vectorize(.help)(x, y)
+}
+
+cset_contains_element <-
+function(x, e)
+{
+    if(cset_is_empty(x))
+        return(FALSE)
+    matchfun <- cset_matchfun(x)
+    x <- .make_list_of_elements_from_cset(x)
+    e <- e(e)
+    if(is.tuple(e) || is.cset(e) || is_element(e))
+        e <- list(e)
+    ind <- matchfun(e, x)
+    if(is.na(ind)) return(FALSE)
+    .get_memberships(x[[ind]]) == .get_memberships(e[[1]])
+}
+
+cset_is_multiset <-
+function(x)
+    gset_is_multiset(x)
+
+cset_is_crisp <-
+cset_is_set_or_multiset <-
+function(x)
+    gset_is_crisp(x)
+
+cset_is_fuzzy_set <-
+function(x)
+    gset_is_fuzzy_set(x)
+
+cset_is_set_or_fuzzy_set <-
+function(x)
+    gset_is_set_or_fuzzy_set(x)
+
+cset_is_fuzzy_multiset <-
+function(x)
+    gset_is_fuzzy_multiset(x)
+
+cset_is_set <-
+function(x)
+    gset_is_set(x)
+
+## contains_element operator dispatch
+
+"%e%" <-
+function(e, x)
+{
+    if(is.set(x))
+        set_contains_element(x, e)
+    else if(is.gset(x))
+        gset_contains_element(x, e)
+    else
+        cset_contains_element(x, e)
+}
+
